﻿

' ===============================================================================
'  This class resizes and reposition all controls on a form.
' ===============================================================================
'
'  INITIALIZATIONS
' ------------------------------------------------------------------------------
'  1. Create a form-level reference to the Resize class:
'     Private myResizer As Resizer = New Resizer
'
'  2. In the Form_Load event, call the Init method:
'     myResizer.Init(Me)
'
'  3. After the "Init" it is possible to disable the font resize for some controls
'     myResizer.DisableFontResize(RichTextBox1)
'     myResizer.DisableFontResize(TextBox2)
'     myResizer.DisableFontResize(GroupBox1)
'
'
'  METHOD 1 - Continuous resize
' ------------------------------------------------------------------------------
'  4. In the Form_Resize event, call the ResizeAllControls method:
'     Private Sub Form1_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
'         myResizer.ResizeAllControls(Me)
'     End Sub
'
'
'  METHOD 2 - Resize controls only if CTRL key is pressed
' ------------------------------------------------------------------------------
'  4. In the Form_Resize event, call the ResizeAllControls method with some conditions:
'     Private Sub Form1_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
'         If My.Computer.Keyboard.CtrlKeyDown AndAlso _
'            My.Computer.Keyboard.ShiftKeyDown AndAlso _
'            Me.WindowState <> FormWindowState.Minimized Then
'                myResizer.ResizeAllControls(Me)
'         End If
'     End Sub
'
'  5. Provide a Button or a Menu-entry to restore the default control dimensions
'      and call the following function:
'       ResetAllControlsToOriginalSize(Me)
'
'
'  METHOD 3 - Resize controls with a trimmer, for example a TrackBar
' ------------------------------------------------------------------------------
'    Private Sub tkbar_Resize_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tkbar_Resize.ValueChanged
'        If Not EventsAreEnabled Then Return
'        Dim ratio As Single = tkbar_Resize.Value
'        ' --------------------------------------- in this example a range 1 .. 9 is changed to 0.6 .. 1.4
'        n = 1 + (n - 5) * 0.1      
'        ResizeAllControls_FixedRatio(Me, ratio)
'    End Sub
'
'
' Warnings:
' -------------------------------------------------------------------------------
'  All controls must have "Autosize" disabled
'
'  When the Form is Maximized or Restored maybe the Resize event is not calld
'  in this case the ResizeAllControls method could be called from the WndProc:
'     Protected Overrides Sub WndProc(ByRef m As Message)
'         Dim old As FormWindowState = Me.WindowState
'         MyBase.WndProc(m)
'         If Me.WindowState <> old Then
'              myResizer.ResizeAllControls(Me)
'         End If
'     End Sub
'
'


Public Class Resizer

    Private MainControlSize_Original As Size
    Private MainControlSize_Resized As Size

    '----------------------------------------------------------
    ' Original state of controls
    '----------------------------------------------------------
    Private Structure ControlInfo
        Public name As String
        Public parentName As String
        Public leftOffsetPercent As Double
        Public topOffsetPercent As Double
        Public heightPercent As Double
        Public originalHeight As Integer
        Public originalWidth As Integer
        Public widthPercent As Double
        Public originalFontSize As Single
        Public fontResize As Boolean
    End Structure

    '-------------------------------------------------------------------------
    ' Dictionary of (control name, control info) for all processed controls
    '-------------------------------------------------------------------------
    Private ctrlDict As Generic.Dictionary(Of String, ControlInfo) = New Generic.Dictionary(Of String, ControlInfo)

    '----------------------------------------------------------------------------------------
    ' Recursive function to process all controls contained in the initially passed
    ' control container and store original position and size in the Control dictionary
    '----------------------------------------------------------------------------------------
    Private Sub FindAllControls_Recursive(ByVal thisCtrl As Control)
        For Each ctl As Control In thisCtrl.Controls
            Try
                If Not IsNothing(ctl.Parent) Then
                    Dim parentHeight As Int32 = ctl.Parent.ClientSize.Height
                    Dim parentWidth As Int32 = ctl.Parent.ClientSize.Width
                    Dim c As New ControlInfo
                    c.name = ctl.Name
                    c.parentName = ctl.Parent.Name
                    c.topOffsetPercent = ctl.Top / parentHeight
                    c.leftOffsetPercent = ctl.Left / parentWidth
                    c.heightPercent = ctl.Height / parentHeight
                    c.widthPercent = ctl.Width / parentWidth
                    c.originalFontSize = ctl.Font.SizeInPoints
                    c.originalHeight = ctl.Height
                    c.originalWidth = ctl.Width
                    c.fontResize = True
                    ctrlDict.Add(c.name, c)
                End If
            Catch ex As Exception
                Debug.Print(ex.Message)
            End Try
            If ctl.Controls.Count > 0 Then
                FindAllControls_Recursive(ctl)
            End If
        Next
    End Sub

    '----------------------------------------------------------------------------------------
    ' INITIALIZATIONS
    '----------------------------------------------------------------------------------------
    Friend Sub Init(ByVal ctrl As Control)
        MainControlSize_Original = ctrl.Size
        FindAllControls_Recursive(ctrl)
    End Sub

    Friend Sub DisableFontResize(ByVal ctrl As Control)
        Dim c As New ControlInfo
        If ctrlDict.TryGetValue(ctrl.Name, c) Then
            c.fontResize = False
            ctrlDict.Item(ctrl.Name) = c
        End If
    End Sub


    '----------------------------------------------------------------------------------------
    ' Recursive function to resize and reposition all controls
    '----------------------------------------------------------------------------------------
    Private Sub ResizeAllControls_Recursive(ByVal ctrl As Control)
        '
        Dim ratioW As Single
        Dim ratioH As Single
        Dim fontRatio As Single
        '
        For Each ctl As Control In ctrl.Controls
            Try
                If Not IsNothing(ctl.Parent) Then
                    Dim ret As Boolean = False
                    Try
                        ' ---------------------------------------------------------- get control info
                        Dim c As New ControlInfo
                        ret = ctrlDict.TryGetValue(ctl.Name, c)
                        ' ---------------------------------------------------------- If found, adjust it
                        If ret Then
                            Dim parentHeight As Int32 = ctl.Parent.ClientSize.Height
                            Dim parentWidth As Int32 = ctl.Parent.ClientSize.Width
                            ' ------------------------------------------------------ size
                            ctl.Width = CInt(parentWidth * c.widthPercent)
                            ctl.Height = CInt(parentHeight * c.heightPercent)
                            ' ------------------------------------------------------ position
                            ctl.Top = CInt(parentHeight * c.topOffsetPercent)
                            ctl.Left = CInt(parentWidth * c.leftOffsetPercent)
                            ' ------------------------------------------------------ font
                            If c.fontResize Then
                                ratioW = CSng(ctl.Width / c.originalWidth)
                                ratioH = CSng(ctl.Height / c.originalHeight)
                                'fontRatio = (ratioW + ratioH) / 2
                                fontRatio = ratioH
                                ctl.Font = New Font(ctl.Font.FontFamily, _
                                                     c.originalFontSize * fontRatio, _
                                                    ctl.Font.Style)
                            End If
                        End If
                    Catch
                    End Try
                End If
            Catch ex As Exception
            End Try
            ' ----------------------------------------------------------------------- recurse for all controls
            If ctl.Controls.Count > 0 Then
                ResizeAllControls_Recursive(ctl)
            End If
        Next
    End Sub


    '----------------------------------------------------------------------------------------
    ' Resize and reposition all controls
    '----------------------------------------------------------------------------------------
    Friend Sub ResizeAllControls(ByVal ctrl As Control)
        MainControlSize_Resized = ctrl.Size
        ResizeAllControls_Recursive(ctrl)
    End Sub

    '----------------------------------------------------------------------------------------
    ' Resize and reposition all controls - FIXED RATIO
    '----------------------------------------------------------------------------------------
    Friend Sub ResizeAllControls_FixedRatio(ByVal ctrl As Control, ByVal ratio As SizeF)
        Dim oldsize As Size = ctrl.Size
        ctrl.Width = CInt(MainControlSize_Original.Width * ratio.Width)
        ctrl.Height = CInt(MainControlSize_Original.Height * ratio.Height)
        ResizeAllControls(ctrl)
        ctrl.Size = oldsize
    End Sub

    '----------------------------------------------------------------------------------------
    ' RESET ALL CONTROLS TO ORIGINAL SIZE and POSITION
    '----------------------------------------------------------------------------------------
    Friend Sub ResetAllControlsToOriginalSize(ByVal ctrl As Control)
        Dim oldsize As Size = ctrl.Size
        ctrl.Width = CInt(MainControlSize_Original.Width)
        ctrl.Height = CInt(MainControlSize_Original.Height)
        ResizeAllControls(ctrl)
        ctrl.Size = oldsize
    End Sub

    '----------------------------------------------------------------------------------------
    ' GET RATIO RESIZED/ORIGINAL (Width and Height)
    '----------------------------------------------------------------------------------------
    Friend Function GetRatio() As SizeF
        GetRatio.Width = CSng(MainControlSize_Resized.Width / MainControlSize_Original.Width)
        GetRatio.Height = CSng(MainControlSize_Resized.Height / MainControlSize_Original.Height)
    End Function

End Class